"use strict";
/*!
 * Contentstack Mongodb Content Connector
 * Copyright (c) 2019 Contentstack LLC
 * MIT Licensed
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Mongodb = void 0;
const debug_1 = __importDefault(require("debug"));
const lodash_1 = require("lodash");
const index_1 = require("./util/index");
const validations_1 = require("./util/validations");
const debug = (0, debug_1.default)("mongodb-core");
let mongo = null;
class Mongodb {
    constructor(mongodb, assetStore, config, appConfig) {
        if (!mongo) {
            this.assetStore = assetStore;
            this.db = mongodb.db;
            this.config = config;
            this.client = mongodb.client;
            this._config = appConfig;
            mongo = this;
        }
    }
    publish(data) {
        return __awaiter(this, void 0, void 0, function* () {
            let response;
            if (data._content_type_uid === "_assets") {
                response = yield this.publishAsset(data);
            }
            else {
                response = yield this.publishEntry(data);
            }
            return response;
        });
    }
    publishAsset(data) {
        debug(`Asset publish called ${JSON.stringify(data)}`);
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            try {
                let assetJSON = (0, lodash_1.cloneDeep)(data);
                (0, validations_1.validateAssetPublish)(assetJSON);
                assetJSON = (0, index_1.filterAssetKeys)(assetJSON);
                if (assetJSON.hasOwnProperty("_version")) {
                    yield this.unpublish(data);
                }
                const asset = yield this.assetStore.download(assetJSON);
                const query = {
                    locale: asset.locale,
                    uid: asset.uid,
                };
                if (asset.hasOwnProperty("download_id")) {
                    query.download_id = asset.download_id;
                }
                else {
                    query._version = asset._version;
                }
                const result = this.db.collection((0, index_1.getCollectionName)(asset)).updateOne(query, {
                    $set: assetJSON,
                }, {
                    upsert: true,
                });
                debug(`Asset publish result ${JSON.stringify(result)}`);
                return resolve(data);
            }
            catch (error) {
                return reject(error);
            }
        }));
    }
    updateContentType(contentType) {
        debug(`Entry publish called ${JSON.stringify(contentType)}`);
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            try {
                let contentTypeJSON = (0, lodash_1.cloneDeep)(contentType);
                (0, validations_1.validateContentTypeUpdate)(contentTypeJSON);
                contentTypeJSON = (0, index_1.filterContentTypeKeys)(contentTypeJSON);
                const contentTypeUpdateResult = yield this.db
                    .collection((0, index_1.getCollectionName)(contentTypeJSON))
                    .updateOne({
                    _content_type_uid: contentTypeJSON._content_type_uid,
                    uid: contentTypeJSON.uid,
                }, {
                    $set: contentTypeJSON,
                }, {
                    upsert: true,
                });
                debug(`Content type update result ${JSON.stringify(contentTypeUpdateResult)}`);
                return resolve(contentType);
            }
            catch (error) {
                return reject(error);
            }
        }));
    }
    publishEntry(entry) {
        debug(`Entry publish called ${JSON.stringify(entry)}`);
        return new Promise((resolve, reject) => {
            var _a, _b, _c, _d;
            try {
                if ((_a = this._config) === null || _a === void 0 ? void 0 : _a.enableContentReferences) {
                    this._updateReferenceFields(entry, (_b = entry._content_type) === null || _b === void 0 ? void 0 : _b.schema, "reference");
                }
                if ((_c = this._config) === null || _c === void 0 ? void 0 : _c.enableAssetReferences) {
                    this._updateAssetFields(entry, (_d = entry._content_type) === null || _d === void 0 ? void 0 : _d.schema, "file");
                }
                let entryJSON = (0, lodash_1.cloneDeep)(entry);
                (0, validations_1.validateEntryPublish)(entryJSON);
                entryJSON = (0, index_1.filterEntryKeys)(entryJSON);
                return this.db
                    .collection((0, index_1.getCollectionName)(entryJSON))
                    .updateOne({
                    _content_type_uid: entryJSON._content_type_uid,
                    locale: entryJSON.locale,
                    uid: entryJSON.uid,
                }, {
                    $set: entryJSON,
                }, {
                    upsert: true,
                })
                    .then((entryPublishResult) => {
                    debug(`Entry publish result ${JSON.stringify(entryPublishResult)}`);
                    return resolve(entry);
                })
                    .catch(reject);
            }
            catch (error) {
                return reject(error);
            }
        });
    }
    unpublish(data) {
        return __awaiter(this, void 0, void 0, function* () {
            let result;
            if (data._content_type_uid === "_assets") {
                result = yield this.unpublishAsset(data);
            }
            else {
                result = yield this.unpublishEntry(data);
            }
            return result;
        });
    }
    delete(data) {
        return new Promise((resolve, reject) => {
            if (data._content_type_uid === "_assets") {
                return this.deleteAsset(data).then(resolve).catch(reject);
            }
            else if (data._content_type_uid === "_content_types") {
                return this.deleteContentType(data).then(resolve).catch(reject);
            }
            return this.deleteEntry(data).then(resolve).catch(reject);
        });
    }
    unpublishEntry(entry) {
        debug(`Delete entry called ${JSON.stringify(entry)}`);
        return new Promise((resolve, reject) => {
            try {
                (0, validations_1.validateEntryRemove)(entry);
                return this.db
                    .collection((0, index_1.getCollectionName)(entry))
                    .deleteOne({
                    _content_type_uid: entry._content_type_uid,
                    locale: entry.locale,
                    uid: entry.uid,
                })
                    .then((result) => {
                    debug(`Delete entry result ${JSON.stringify(result)}`);
                    return resolve(entry);
                })
                    .catch(reject);
            }
            catch (error) {
                return reject(error);
            }
        });
    }
    deleteEntry(entry) {
        debug(`Delete entry called ${JSON.stringify(entry)}`);
        return new Promise((resolve, reject) => {
            try {
                (0, validations_1.validateEntryRemove)(entry);
                return this.db
                    .collection((0, index_1.getCollectionName)(entry))
                    .deleteMany({
                    _content_type_uid: entry._content_type_uid,
                    uid: entry.uid,
                })
                    .then((result) => {
                    debug(`Delete entry result ${JSON.stringify(result)}`);
                    return resolve(entry);
                })
                    .catch(reject);
            }
            catch (error) {
                return reject(error);
            }
        });
    }
    unpublishAsset(asset) {
        debug(`Unpublish asset called ${JSON.stringify(asset)}`);
        return new Promise((resolve, reject) => {
            try {
                (0, validations_1.validateAssetUnpublish)(asset);
                return this.db
                    .collection((0, index_1.getCollectionName)(asset))
                    .findOneAndDelete({
                    _content_type_uid: asset._content_type_uid,
                    _version: {
                        $exists: true,
                    },
                    locale: asset.locale,
                    uid: asset.uid,
                })
                    .then((result) => {
                    debug(`Asset unpublish status: ${JSON.stringify(result)}`);
                    if (!(result === null || result === void 0 ? void 0 : result.value)) {
                        return resolve(asset);
                    }
                    return this.db
                        .collection((0, index_1.getCollectionName)(asset))
                        .find({
                        _content_type_uid: asset._content_type_uid,
                        download_id: {
                            $exists: true,
                        },
                        locale: asset.locale,
                        uid: asset.uid,
                        url: result.url,
                    })
                        .toArray()
                        .then((assets) => {
                        if (assets.length === 0) {
                            debug(`Only published object of ${JSON.stringify(asset)} was present`);
                            return this.assetStore
                                .unpublish(result.value)
                                .then(() => resolve(asset));
                        }
                        debug("Asset existed in pubilshed and RTE/Markdown form. Removed published asset object.");
                        return resolve(asset);
                    });
                })
                    .catch(reject);
            }
            catch (error) {
                return reject(error);
            }
        });
    }
    deleteAsset(asset) {
        debug(`Delete asset called ${JSON.stringify(asset)}`);
        return new Promise((resolve, reject) => {
            try {
                (0, validations_1.validateAssetDelete)(asset);
                return this.db
                    .collection((0, index_1.getCollectionName)(asset))
                    .find({
                    _content_type_uid: "_assets",
                    locale: asset.locale,
                    uid: asset.uid,
                })
                    .toArray()
                    .then((result) => {
                    if (result.length === 0) {
                        debug("Asset did not exist!");
                        return resolve(asset);
                    }
                    return this.db
                        .collection((0, index_1.getCollectionName)(asset))
                        .deleteMany({
                        _content_type_uid: "_assets",
                        locale: asset.locale,
                        uid: asset.uid,
                    })
                        .then(() => {
                        return result;
                    });
                })
                    .then((result) => {
                    return this.assetStore.delete(result).then(() => resolve(asset));
                })
                    .catch(reject);
            }
            catch (error) {
                return reject(error);
            }
        });
    }
    deleteContentType(contentType) {
        debug(`Delete content type called ${JSON.stringify(contentType)}`);
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            try {
                (0, validations_1.validateContentTypeDelete)(contentType);
                const collectionsResult = yield this.db
                    .listCollections({}, { nameOnly: true })
                    .toArray();
                if (collectionsResult.length === 0) {
                    return resolve(contentType);
                }
                const collections = (0, index_1.getLocalesFromCollections)(collectionsResult);
                const promisifiedBucket = [];
                collections.forEach((collection) => {
                    promisifiedBucket.push(this.deleteCT(contentType.uid, collection));
                });
                return Promise.all(promisifiedBucket).then(() => resolve(contentType));
            }
            catch (error) {
                return reject(error);
            }
        }));
    }
    deleteCT(uid, collection) {
        return new Promise((resolve, reject) => {
            try {
                return this.db
                    .collection((0, index_1.getCollectionName)({
                    _content_type_uid: uid,
                    locale: collection.locale,
                }))
                    .deleteMany({
                    _content_type_uid: uid,
                })
                    .then((entriesDeleteResult) => {
                    debug(`Delete entries result ${JSON.stringify(entriesDeleteResult)}`);
                    return this.db
                        .collection(collection.name)
                        .deleteOne({
                        _content_type_uid: "_content_types",
                        uid,
                    })
                        .then((contentTypeDeleteResult) => {
                        debug(`Content type delete result ${JSON.stringify(contentTypeDeleteResult)}`);
                        return resolve(0);
                    });
                })
                    .catch(reject);
            }
            catch (error) {
                return reject(error);
            }
        });
    }
    _updateReferenceFields(data, ctSchema, fieldType) {
        var _a, _b, _c;
        for (const field of ctSchema) {
            if (field.data_type === fieldType) {
                data[field.uid] = {
                    reference_to: (_a = field.reference_to) === null || _a === void 0 ? void 0 : _a[0],
                    value: (_c = (_b = data[field.uid]) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.uid,
                };
            }
            if (field.schema) {
                this._updateReferenceFields(data[field.uid], field.schema, fieldType);
            }
        }
    }
    _updateAssetFields(data, ctSchema, fieldType) {
        for (const field of ctSchema) {
            if (field.data_type === fieldType) {
                data[field.uid] = {
                    reference_to: "_assets",
                    value: data[field.uid],
                };
            }
            if (field.schema) {
                this._updateAssetFields(data[field.uid], field.schema, fieldType);
            }
        }
    }
}
exports.Mongodb = Mongodb;
